﻿//BERNARDO DEL CASTILLO - RAPTUS GAMES 2014
using UnityEngine;
using System.Collections;

public class CompSimple : MonoBehaviour {
	public Camera lightCam;
	private RenderTexture LightRender, CompLight, BlurVert, BlurHoriz, depthBase, depthEffect;

	public Shader ComposeLight, GaussianBlur;
	public Material LightComp, GenericBlur;

	private int ScreenX = 1280, ScreenY = 720;
	public int blurIterations = 1;
	public float blurIntensity;

	RenderTextureFormat RTT_Format;


	public static CompSimple _stImageComposer;

	private enum passes{
		compose,
		interpretDepth,
		correctCulling
	}
	
	// Use this for initialization
	void OnEnable()
	{

		_stImageComposer = this;
		LightComp = new Material(ComposeLight);
		GenericBlur = new Material (GaussianBlur);


		TextureFormat();
		camera.depthTextureMode = DepthTextureMode.Depth;
		lightCam.depthTextureMode = DepthTextureMode.Depth;

		CompLight = new RenderTexture (Screen.width, Screen.height, 16, RTT_Format);
		BlurVert = new RenderTexture (Screen.width, Screen.height, 16, RTT_Format);
		BlurHoriz = new RenderTexture (Screen.width, Screen.height, 16, RTT_Format);

		depthEffect = new RenderTexture (Screen.width, Screen.height, 16, RenderTextureFormat.ARGBHalf);
		depthBase = new RenderTexture (Screen.width, Screen.height, 16, RenderTextureFormat.ARGBHalf);

		lightCam.targetTexture = new RenderTexture(Screen.width, Screen.height, 16, RTT_Format);
	}

	void TextureFormat()
	{
		//Default Unity formats
		
		if (Camera.main.hdr)
			RTT_Format = RenderTextureFormat.ARGBHalf;
		else
			RTT_Format = RenderTextureFormat.ARGB32;
	}
	
	void OnRenderImage (RenderTexture source, RenderTexture destination)
	{
		#if UNITY_EDITOR
		TextureFormat();
		#endif

		//We set the values to the actual size of the render texture

		ScreenX = source.width;
		ScreenY = source.height;

		//GENERATE FIRST DEPTH TEXTURE
		Graphics.Blit (source, depthBase, LightComp, (int)passes.interpretDepth);

		// SET LIGHT TEXTURE
		lightCam.Render ();
		LightRender = lightCam.targetTexture;
		LightRender.name = "LightRender";
		CompLight.name = "GeomSampler";

		//GENERATE SECOND DEPTH TEXTURE
		Graphics.Blit (source, depthEffect, LightComp, (int)passes.interpretDepth);

		//ADD THE TEXTURES
		LightComp.SetTexture ("_BaseDepth", depthBase);
		LightComp.SetTexture ("_LightDepth", depthEffect);

		LightComp.SetTexture ("_LightTex", LightRender);

		Graphics.Blit (source, CompLight, LightComp, (int)passes.correctCulling);

		//APPLY RENDERED LIGHT MESH TO GAUSSIAN BLUR MATERIAL
		blurFunction ();

		//SET THE COMPLETED BLUR AS THE LIGHT TEXTURE TO COMPOSE

		LightComp.SetTexture ("_LightTex", BlurVert);

		Graphics.Blit (source, destination, LightComp, (int)passes.compose);

		//RenderTexture.active = destination;

		
	}

	void blurFunction()
	{
		
		GenericBlur.SetTexture ("_IMGtoBlur", CompLight);
		GenericBlur.SetFloat ("_Blur", blurIntensity);
		
		int iter = 0;
		
		//GAUSSIAN BLUR ITERATIONS
		
		while (iter < blurIterations) {
			//MAKE AN HORIZONTAL PASS
			GenericBlur.SetInt ("_Horiz", 1);
			Graphics.Blit ( CompLight, BlurHoriz, GenericBlur, 0);
			
			//SET THE HORIZONTAL BLURED IMAGE BACK AS THE SOURCE
			GenericBlur.SetTexture ("_IMGtoBlur", BlurHoriz);
			
			//MAKE A VERTICAL BLUR PASS
			GenericBlur.SetInt ("_Horiz", 0);
			Graphics.Blit ( CompLight, BlurVert, GenericBlur, 0);
			
			//SET THE COMPLETED BLUR BACK AS THE SOURCE
			GenericBlur.SetTexture ("_IMGtoBlur", BlurVert);
			
			iter ++;
		}
	}
}
