﻿//BERNARDO DEL CASTILLO - RAPTUS GAMES 2014
Shader "Custom/SimpleCompose" {
	Properties {
		_MainTex ("Base (RGB)", 2D) = "white" {}
	}
	
	CGINCLUDE
	#include "UnityCG.cginc"
	
	uniform sampler2D		_MainTex,
							_LightTex,
							_BaseDepth,
							_LightDepth,
							_CameraDepthTexture;
	
	uniform half4 			_MainTex_TexelSize;
							
	struct v2f {
		half4 pos : POSITION;
		half2 uv : TEXCOORD0;
		half4 scrPos : TEXCOORD1;
		
	};
	
	//Common Vertex Shader
	v2f vert( appdata_img v )
	{
		v2f o;
		o.pos = mul (UNITY_MATRIX_MVP, v.vertex);
		o.uv = v.texcoord;
		o.scrPos=ComputeScreenPos(o.pos);
		
		
		//o.scrPos.y = 1 - o.scrPos.y;
		return o;
	} 
	
	half4 GetDepth(v2f IN) : COLOR
	{
		half depthValue = Linear01Depth (tex2Dproj(_CameraDepthTexture, UNITY_PROJ_COORD(IN.scrPos)).r);
		half4 depth;

		depth.r = depthValue;
		depth.g = depthValue;
		depth.b = depthValue;

		depth.a = 0;

		return depth;
	}
	
	half4 BlurDepth(half4 depthBase, half4 depthLight, half4 lightSample, half4 baseSample)
	{
		if(depthBase.r <= depthLight.r ||  depthLight.r >= 0.98)
		{
		    //Effect diffusing the unlit areas ith brightness bloom 
			return  half4(baseSample.r + lightSample.r,baseSample.g + lightSample.g,baseSample.b +lightSample.b,0);
			
			//CAN USE THE FOLLOWING IF YOU DONT LIKE THE RESULT THERE
			//DARKER BLOOM CONSIDERING UNLIT COLOR
			//return  half4(baseSample.r ,baseSample.g ,baseSample.b,0);
			//IGNORING BASE COLOR
			//return  half4(0,0,0,0);
		}
		else
		{
			return lightSample;
		}
	}
	
	half4 CorrectWithDepth(v2f IN) : COLOR
	{		
	
		half2 texcoords = IN.uv;
		
		half2 Temptexcoords = texcoords;
		#if UNITY_UV_STARTS_AT_TOP
			if (_MainTex_TexelSize.y < 0)
		        Temptexcoords.y = 1-Temptexcoords.y;
		#endif
		
		
		half4 LightSample = tex2D(_LightTex, Temptexcoords);
		half4 LightDepth = tex2D(_LightDepth, Temptexcoords);
		
		half4 BaseDepth = tex2D(_BaseDepth, Temptexcoords);
		half4 BaseSample = tex2D(_MainTex, texcoords);
		
		half4 finalBlurSample = half4(0,0,0,0);
		
		finalBlurSample = BlurDepth(BaseDepth, LightDepth, LightSample, BaseSample);
		
		return finalBlurSample;
	}
	
	half4 Compose(v2f IN) : COLOR
	{
		half4 Final;
		
		half2 texcoords = IN.uv;
		
		//FIX WHEN UNITY DOES ANTIALIASING
		half2 Temptexcoords = texcoords;
		#if UNITY_UV_STARTS_AT_TOP
			if (_MainTex_TexelSize.y < 0)
		        Temptexcoords.y = 1-Temptexcoords.y;
		#endif
		
		half4 finalLightSample = tex2D(_LightTex, Temptexcoords);
		half4 Scene = tex2D(_MainTex, texcoords);
		
		half ColCoef = length (finalLightSample.a);
		Final = (Scene*(1-ColCoef) +  finalLightSample * ColCoef);
		
		return Final;
	}
	
	
	ENDCG
	
	SubShader {
		ZTest Off
		Cull Off
		ZWrite Off
		Fog { Mode off }
		
		Pass 
		 {
			 Name "Compose"

			      CGPROGRAM
			      #pragma target 4.0
				  #pragma fragmentoption ARB_precision_hint_fastest
			      #pragma vertex vert
			      #pragma fragment Compose
			      ENDCG
		 }
		 
		Pass 
		 {
			 Name "GetDepth"

			      CGPROGRAM
			      #pragma target 4.0
				  #pragma fragmentoption ARB_precision_hint_fastest
			      #pragma vertex vert
			      #pragma fragment GetDepth
			      ENDCG
		 }
		 
		 Pass 
		 {
			 Name "CorrectWithDepth"

			      CGPROGRAM
			      #pragma target 4.0
				  #pragma fragmentoption ARB_precision_hint_fastest
			      #pragma vertex vert
			      #pragma fragment CorrectWithDepth
			      ENDCG
		 }
		 
	} 
	FallBack off
}
